@@ -32,6 +32,20 @@ impl<'a, N: Node> Step<'a, N> {
             Step::Unresolved(_) => false,
         }
     }
+
+    pub fn as_resolved(&self) -> Option<&N> {
+        match self {
+            Step::Resolved(node) => Some(node),
+            Step::Unresolved(_) => None,
+        }
+    }
+
+    pub fn as_unresolved(&self) -> Option<&N::DependencyType> {
+        match self {
+            Step::Resolved(_) => None,
+            Step::Unresolved(dependency) => Some(dependency),
+        }
+    }
 }
 
 /// The [`DependencyGraph`] structure builds an internal [Directed Graph](`petgraph::stable_graph::StableDiGraph`), which can then be traversed
@@ -86,6 +100,13 @@ where
     pub fn is_internally_resolvable(&self) -> bool {
         self.graph.node_weights().all(Step::is_resolved)
     }
+
+    /// Get an iterator over unresolved dependencies, without traversing the whole graph.
+    /// Useful for doing pre-validation or pre-fetching of external dependencies before
+    /// starting to resolve internal dependencies.
+    pub fn unresolved_dependencies(&self) -> impl Iterator<Item = &N::DependencyType> {
+        self.graph.node_weights().filter_map(Step::as_unresolved)
+    }
 }
 
 /// Iterate over the DependencyGraph in an order which ensures dependencies are resolved before each Node is visited.
@@ -161,6 +182,21 @@ mod tests {
         }
     }
 
+    #[test]
+    fn test_unresolved_dependencies() {
+        let build = build_test_graph();
+        let graph = DependencyGraph::from(&build[..]);
+
+        assert!(!graph.is_internally_resolvable());
+
+        let unresolved_dependencies: Vec<_> = graph
+            .unresolved_dependencies()
+            .map(|dep| dep.name)
+            .collect();
+
+        assert_eq!(unresolved_dependencies, vec!["unknown", "remote"]);
+    }
+
     #[test]
     fn test_generate_dependency_graph() {
         DependencyGraph::from(&build_test_graph()[..]);
@@ -247,10 +283,16 @@ mod tests {
                     pre: Prerelease::new("").unwrap(),
                     build: BuildMetadata::EMPTY,
                 },
-                dependencies: vec![Dependency {
-                    name: "unknown",
-                    version: ">=1.0.0".parse().unwrap(),
-                }],
+                dependencies: vec![
+                    Dependency {
+                        name: "unknown",
+                        version: ">=1.0.0".parse().unwrap(),
+                    },
+                    Dependency {
+                        name: "remote",
+                        version: "=3.0.0".parse().unwrap(),
+                    },
+                ],
             },
         ]
     }
